<?php

namespace Controller;

use Service\Traits\TResponse;
use Service\DataSourceService;
use Service\DefaultColumns;
use Service\DeleteCascadeService;
use Util\MongoDataBaseAlterations;
use Zmart\Mongo\Mongo;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\Manager;
use Doctrine\DBAL\Connection;
use Util\Period;
use Util\InstanceProvider;
use Zeedhi\Framework\DataSource\Manager\Mongo\Types\Type;
use Util\Environment;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Util\Utilities;
use Moment\Moment;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

class DataSource extends Crud
{

    use TResponse;

    const ERRORS_CONNECT_FAIL = "oci_connect()";
    const ERRORS_EXECUTE_QUERY_FAIL = "oci_execute()";
    const BOOLEAN_FIELDS = [
        "hasDataMining",
        "isScheduleActive",
        "willUpdateIndicators"
    ];

    protected $dataSourceName = 'dataSource';

    /** @var Connection */
    protected $connection;

    protected $dataSourceService;

    protected $dataSourceOtherService;

    protected $importHistoryService;

    protected $importScheduleService;

    protected $socketClient;

    protected $environment;
    /** @var \Service\Manager\Mongo\Manager  */
    protected $organizationMongoManager;

    public function __construct(
        Manager $organizationMongoManager,
        DeleteCascadeService $deleteCascadeService,
        DataSourceService $dataSourceService,
        \Service\DataSource $dataSourceOtherService,
        \Service\ImportHistory $importHistory,
        \Service\ImportSchedule $importSchedule,
        \Service\SocketClient $socketClient,
        Environment $environment
    ) {

        $this->dataSourceService        = $dataSourceService;
        $this->dataSourceOtherService   = $dataSourceOtherService;
        $this->importHistoryService     = $importHistory;
        $this->importScheduleService    = $importSchedule;
        $this->socketClient             = $socketClient;
        $this->organizationMongoManager = $organizationMongoManager;
        $this->environment              = $environment;
        $this->organizationMongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        parent::__construct($organizationMongoManager, $deleteCascadeService);
    }

    public function scheduleMiliseconds(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $rows = $this->mongoManager->findBy($filterData)->getRows();
        $miliseconds = -1;
        if (!empty($rows)) {
            $rows = $rows[0];
            if (!$rows["hasDataMining"] && $rows["willUpdateIndicators"] && !empty($rows["scheduleExpression"])) {
                $scheduleExpression = $rows["scheduleExpression"];
                $now = new \DateTime();
                $then = Period::getNextRunDateFromExpression($scheduleExpression);
                $miliseconds = 1000 * ($then->getTimestamp() - $now->getTimestamp());
            }
        }
        $response->addDataSet(new DataSet("dataset", [$miliseconds]));
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        Utilities::convertDynamicFilterToArray($params);
        Utilities::convertBoolFilterToBoolean($params, ['hasDataMining', 'joinDataSource', 'willUpdateIndicators', 'isScheduleActive']);

        $rows = $this->mongoManager->findNext($this->dataSourceName, $params, false);
        $this->addHistoryDataToRows($rows);
        foreach ($rows as &$row) {
            foreach ($row["columns"] as &$column) {
                if (!empty($column["isDate"]) && $column["isDate"]) {
                    $column["datatype"] = "D";
                } else if (empty($column["datatype"])) {
                    $column["datatype"] = "A";
                }
            }
        }
        $this->convertBooleanValues($rows, ['hasDataMining', 'joinDataSource', 'willUpdateIndicators', 'isScheduleActive']);
        $data = Utilities::parseDataSourceZeedhiNext($rows, $this->count($params, false));
        $data["pagination"]["page"] = array_key_exists("page", $params) ? intval($params["page"]) : 1;
        $data["pagination"]["limit"] = array_key_exists("limit", $params) ? intval($params["limit"]) : 9999;
        $this->sendResponse($data, 200);
    }

    public function validationIsTeknisa($row, $dataBefore, $response)
    {
        if(!Utilities::userCurrentIsTeknisa() && $row['IS_TEKNISA'] == 'Y'){
            if (!empty($dataBefore) && $dataBefore[0]['query'] != $row['query']) {
                $response->addMessage(new Message('A query enviada é diferente da que se encontra no banco e ela só pode ser atualizada por operadores teknisa', Message::TYPE_ERROR));
            }
            if (!empty($row['__is_new'])){
                $response->addMessage(new Message('Somente operadores teknisa podem adicionar fonte de dados com essa conexão de banco de dados!', Message::TYPE_ERROR));
            }
        }

    }

    public function convertBooleanValues(&$data, $columns)
    {
        foreach ($data as &$row) {
            foreach ($columns as &$column) {
                if (array_key_exists($column, $row)) {
                    $row[$column] = $row[$column] ? "S" : "N";
                }
            }
        }
    }

    public function revertToBooleanValues(&$data, $columns)
    {
        $rows = is_array($data) ? $data : [$data];
        foreach ($rows as &$row) {
            foreach ($columns as &$column) {
                if (array_key_exists($column, $row)) {
                    $row[$column] = $row[$column] != "N";
                }
            }
        }
    }

    public function count($params, $decode = true)
    {
        return $this->mongoManager->countNext($this->dataSourceName, $params, $decode);
    }

    public function save(DTO\Request $request, DTO\Response $response)
    {
        $columnsRows = array();
        $relationRows = array();
        $request = $this->buildDataSetIfRow($request);
        $rows = $request->getDataSet()->getRows();
        $this->revertToBooleanValues($rows, ['hasDataMining', 'joinDataSource', 'willUpdateIndicators', 'isScheduleActive']);
        foreach ($rows as &$row) {
            $dataBefore = null;
            if(!empty($row['_id'])){
                $dataBefore = $this->dataSourceOtherService->findById($row['_id']);
            }
            if(empty($row['IS_TEKNISA'])){
               $row['IS_TEKNISA'] = 'N';
            }
            if($row['IS_TEKNISA'] != 'Y' && !empty($row['connection'])){
                $connection = InstanceProvider::getControllerConnection()->findById($row['connection']);
                if(!empty($connection[0]["IS_TEKNISA"]) && $connection[0]["IS_TEKNISA"] == 'Y'){
                    $row['IS_TEKNISA'] = 'Y';
                }
            }
            $this->validationIsTeknisa($row, $dataBefore, $response);
            $this->setIsKeyColumns($row);
            $this->cleanRowFields($row);
            foreach ($row['columns'] as &$col) {
                if ((!isset($col['expression']) || !$col['expression']) && isset($col['mathExpressions'])) {
                    unset($col['mathExpressions']);
                }
                if (!empty($col["datatype"]) && $col["datatype"] == "D") {
                    $col["isDate"] = true;
                } else {
                    $col["isDate"] = false;
                }
                if (!empty($col['isFact']) && !empty($col['specialFunction'])) {
                    if ($col['specialFunction'] == 'case' && !empty($col['relatedFactLabel'])) {
                        unset($col['relatedFactLabel']);
                    } else if ($col['specialFunction'] != 'case' && !empty($col['relatedFactMultiple'])) {
                        unset($col['relatedFactMultiple']);
                        if (!empty($col['relatedFactMultipleLabel'])) {
                            unset($col['relatedFactMultipleLabel']);
                        }
                    }
                }
            }
            if ((isset($row["_queryChanged"]) && $row["_queryChanged"]) || (isset($row["_originDatasourcesChanged"]) && $row["_originDatasourcesChanged"])) {
                try {
                    $row["columns"] = (isset($row["joinDataSource"]) && $row["joinDataSource"]) ? $this->getColumnsFromOriginDataSources($row) : $this->getColumnsFromWebSocket($row);
                    $columnsRows[] = array("dataSource" => $row['name'], "columns" => $row['columns']);
                    $row["relatedStructuresType"] = $this->dataSourceService->mergeDefaultRelations($row);
                    $relationRows[] = array("dataSource" => $row['name'], "relations" => $row['relatedStructuresType']);

                    $this->unsetChangeControlVariables($row);
                } catch (Exception $e) {
                    if ($e->getCode() === Exception::CODE_FAIL_CONNECTION || $e->getCode() === Exception::CODE_QUERY_EXECUTION_FAIL) {
                        $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
                    } else {
                        throw $e;
                    }
                }
            } else {
                try {
                    $this->dataSourceService->insertNewDefaultColumns($row["columns"]);
                    if (isset($row['relatedStructuresType'])) {
                        $this->dataSourceService->insertNewDefaultRelations($row['relatedStructuresType']);
                    }
                } catch (Exception $e) {
                    throw $e;
                }
            }

            if (!empty($row["_queryChanged"]) || !empty($row["_columnsChanged"])) {
                if (isset($row["modified"])) {
                    $row["modified"] = true;
                }
                if (Utilities::getOrganizationIdForMongo() == "5") {
                    $message = $this->dataSourceService->changeDataSourcesWithOldId($row);
                    if (!empty($message)) {
                        $response->addDataSet(new DataSet('message', $message));
                        return;
                    }
                }
            }

            if (isset($row["joinDataSource"]) && $row["joinDataSource"]) {
                $this->setFactsJoinFunctionsInformations($row);
                $this->setAutoFactsTranslation($row);
            }
            if (isset($row["isScheduleSave"])) {
                $isScheduleSave = $row["isScheduleSave"];
                $isScheduleActive = isset($row['isScheduleActive']) ? $row['isScheduleActive'] : false;
                unset($row["isScheduleSave"]);
                if ($isScheduleSave === true) {
                    try {
                        $this->deleteOldScheduleItems($row);
                        if ($isScheduleActive) {
                            $this->saveScheduleItem($row);
                        }
                    } catch (\InvalidArgumentException $e) {
                        $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
                        return;
                    }
                }
            }
            if (!empty($row["confirm"])) {
                unset($row["confirm"]);
            }
        }

        if (isset($row["isEdited"]) && $row["isEdited"] && !empty($row["dateFilterColumn"])) {
            $this->saveDateFilterColumnIndexes($row);
        }

        Utilities::clearArrayItemsKeys($row, ["_queryChanged", "_columnsChanged"]);
        parent::save($request, $response);
        $id = $response->getDataSets()[0]->getRows()[0]['_id'];
        $this->dataSourceOtherService->setLog($dataBefore, $id);

        $response->addDataSet(new DataSet('columns', $columnsRows));
        $response->addDataSet(new DataSet('relations', $relationRows));
    }

    protected function saveDateFilterColumnIndexes($row)
    {
        try {
            try {
                $this->dataSourceOtherService->saveIndexes($row["_id"], $row["dateFilterColumn"], [$row["dateFilterColumn"]], [1]);
            } catch (\Exception $e) {
                if ($e->getCode() == 86) {
                    $this->dataSourceOtherService->saveIndexes($row["_id"], $row["dateFilterColumn"] . "_ASC", [$row["dateFilterColumn"]], [1]);
                }
            }
            $this->dataSourceOtherService->saveIndexes($row["_id"], $row["dateFilterColumn"] . "_DESC", [$row["dateFilterColumn"]], [-1]);
        } catch (\Exception $e) {
        }
    }

    protected function unsetChangeControlVariables(&$row)
    {
        $flags = ['_queryChanged', '_originDatasourcesChanged', '_queryBkp'];
        foreach ($flags as $flag) {
            if (isset($row[$flag])) {
                unset($row[$flag]);
            }
        }
    }

    protected function saveScheduleItem($row)
    {
        if (!empty($row['_id'])) {
            $schedule = array();
            $this->setScheduleItems($row, $schedule);
            $schedule['date'] = Period::getNextRunDateFromExpression($schedule['scheduleExpression']);
            $schedule['date'] = Type::getType('mongoDate')->convertToDatabaseValue($schedule['date']);
            return $this->importScheduleService->saveOne($schedule);
        }
    }

    protected function setScheduleItems($row, &$schedule)
    {
        $items = array('period', 'scheduleExpression', 'beforeImportScript', 'afterImportScript', 'persistType');
        foreach ($items as $item) {
            $schedule[$item] = isset($row[$item]) ? $row[$item] : null;
        }
        $schedule['dataSource'] = $row['_id'];
        $schedule['type']       =  $row['type'];
    }

    protected function deleteOldScheduleItems($row)
    {
        if (!empty($row['_id'])) {
            $scheduleItemsByDataSourceId = $this->importScheduleService->findByDataSourceId($row['_id']);
            $removeList = array();
            foreach ($scheduleItemsByDataSourceId as $scheduleItem) {
                if (!empty($scheduleItem['scheduleExpression'])) {
                    $removeList[] = $scheduleItem;
                }
            }

            if (!empty($removeList)) {
                $this->importScheduleService->delete($removeList);
            }
        }
    }

    public function findByIndicator(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $indicatorId = $conditions[0]['value'];
        $dataSource = $this->dataSourceOtherService->findByIndicator($indicatorId);
        $dataSet = new DataSet($this->dataSourceName, array($dataSource));
        $response->addDataSet($dataSet);
    }

    public function findDataSourceGeneratedData(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        $dataSourceId = null;

        foreach ($params as $key => $condition) {
            if ($key == "dataSourceId") {
                $dataSourceId = $condition;
                unset($params[$key]);
            }
        }
        
        $filterData = new FilterCriteria($this->dataSourceName);
        $filterData->addCondition('_id',$dataSourceId);
        $rows = $this->mongoManager->findBy($filterData)->getRows();

        $columns = null;
        if(isset($rows[0])){
            $columns = $rows[0]["columns"];
        }
        
        $rows = [];
        $dataSet = $this->mongoManager->findNext($dataSourceId, $params);

        if ($dataSet == NULL) {
            $this->sendResponse(NULL, 200);
        } else {
            foreach ($dataSet as &$row) {
                foreach ($row as $name => $data) {
                    if ($data instanceof \DateTime) {
                        $row[$name] = $data->format('d/m/y');
                    }
                }

                array_push($rows, $row);
            }
            $data = Utilities::parseDataSourceZeedhiNext($rows);
            // Paginação
            $data["pagination"]["total"] = $this->countDataSourceGeneratedData($params, $dataSourceId);
            $data["pagination"]["page"] = array_key_exists("page", $params) ? intval($params["page"]) : 1;
            $data["pagination"]["limit"] = array_key_exists("limit", $params) ? intval($params["limit"]) : 9999;
            $this->sendResponse($data, 200);
        }
    }

    public function countDataSourceGeneratedData($params, $dataSourceName)
    {
        return $this->mongoManager->countNext($dataSourceName, $params);
    }

    public function getIndexes(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        $filter = Utilities::parseParamNextToZmartFilterCriteria("indexes", $params)->getConditions();
        $dataSource = Utilities::getValueInFilterDataSourceByName($filter, "dataSourceId");
        $indexes = $this->dataSourceOtherService->getIndexes($dataSource);
        $data = Utilities::parseDataSourceZeedhiNext($indexes);
        $data["pagination"]["page"] = 1;
        $data["pagination"]["limit"] = 9999;
        $this->sendResponse($data, 200);
        //$response->addDataSet(new DataSet($t
    }

    public function deleteIndexes(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this->dataSourceOtherService->deleteIndexes($row);
        $dataSource = explode('.', $row[0]['ns'])[1];
        $this->dataSourceOtherService->setlogIndex(array($row), null, $dataSource);
    }

    public function checkIndicatorsWithDataSource(DTO\Request $request, DTO\Response $response){
        try {
            $rows = $request->getDataSet()->getRows();
            $this->dataSourceOtherService->checkIndicatorsWithDataSource($rows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];
        $this->dataSourceOtherService->checkIndicatorsWithDataSource([$row]);
        $dataBefore = null;
        if (!empty($row['_id'])) {
            $dataBefore = $this->dataSourceOtherService->findById($row['_id']);
        }
        parent::delete($request, $response);
        $this->dataSourceOtherService->setLog($dataBefore, $row['_id']);
    }

    public function saveIndexes(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $columns = $row["COLUMN_"];
            $orders = $row["ORDER_"];
            $name = $row["name"];
            $dataSourceId = $row["dataSource"];
            if (Utilities::getOrganizationIdForMongo() == "5") {
                $dataSource = $this->dataSourceOtherService->findById($dataSourceId)[0];
                $dataSource["confirm"] = !empty($row["confirm"]) ? $row["confirm"] : null;
                $message = $this->dataSourceService->changeDataSourcesWithOldId($dataSource, $row);
                if (!empty($message)) {
                    $response->addDataSet(new DataSet('message', $message));
                    return;
                }
            }
            $this->dataSourceOtherService->saveIndexes($dataSourceId, $name, $columns, $orders);
            $this->dataSourceOtherService->setlogIndex(null, array($row), $dataSourceId);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    public function findImportProcesses(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $dataSourceId = $conditions[0]['value'];

        $numberImportHistory = count($this->importHistoryService->findImportsInProgress($dataSourceId));
        $numberImportSchedule = count($this->importScheduleService->findImportsInProgress($dataSourceId));
        $response->addDataSet(new DataSet('totalImportsInProcess', array($numberImportHistory + $numberImportSchedule)));

        $processesBlocked = count($this->importHistoryService->findImportsInProgressBlocked($dataSourceId));
        $response->addDataSet(new DataSet('totalImportsBlocked', array($processesBlocked)));
    }

    public function updateStatusSchedule(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $dataSourceId = $conditions[0]['value'];

        $importSchedule = $this->importScheduleService->findByDataSourceId($dataSourceId);
        $newValue = 'queue';

        $importSchedule = $this->importScheduleService->updateImportSchedule($importSchedule, $newValue);
    }

    public function cancelImportProcesses(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $dataSourceId = $conditions[0]['value'];

        $importsHistory = $this->importHistoryService->findImportsInProgress($dataSourceId);

        foreach ($importsHistory as &$history) {
            $history['importStatus'] = 'canceled';
            $history['createDate'] = Type::getType('mongoDate')->convertToDatabaseValue($history['createDate']);
            $history['endDate'] = new \MongoDB\BSON\UTCDateTime((new Moment(null, 'America/Sao_Paulo'))->format('U') . '000');
        }
        $this->importHistoryService->updateImports($importsHistory);

        $importSchedule = $this->importScheduleService->findImportsInProgress($dataSourceId);
        $this->importScheduleService->delete($importSchedule);
    }

    public function clearAllRelations(DTO\Request\Row $request, DTO\Response $response)
    {
        $dataSource = $request->getRow()["dataSource"];
        $row = $this->dataSourceOtherService->clearAllRelations($dataSource);
        $dataSet = $this->organizationMongoManager->persist(new DataSet($this->dataSourceName, array($row[0])));
        $response->addDataSet(new DataSet("clearAllRelations", [$dataSet]));
    }

    public function syncColumns(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this->revertToBooleanValues($row, ['hasDataMining', 'joinDataSource', 'willUpdateIndicators', 'isScheduleActive']);
        $this->cleanRowFields($row);
        $columns = array();

        try {
            if (isset($row['joinDataSource']) && $row['joinDataSource']) {
                $columns = $this->getColumnsFromOriginDataSources($row);
            } else {
                $columns = $this->getColumnsFromWebSocket($row);
            }

            $dataSet = new DataSet("columns", $columns);
            $response->addDataSet($dataSet);
            $row['columns'] = $columns;
            $this->unsetChangeControlVariables($row);
            $this->mongoManager->persist(new DataSet($this->dataSourceName, array($row)));
        } catch (Exception $e) {
            if ($e->getCode() === Exception::CODE_FAIL_CONNECTION || $e->getCode() === Exception::CODE_QUERY_EXECUTION_FAIL) {
                $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
            } else {
                throw $e;
            }
        }
    }

    protected function cleanRowFields(&$row)
    {
        if (isset($row['importStatus'])) {
            unset($row['importStatus']);
        }
        if (isset($row['importHasError'])) {
            unset($row['importHasError']);
        }
        /*
        if (empty($row['_id'])) {
            unset($row['_id']);
        }
        */
    }

    protected function setIsKeyColumns(&$row)
    {
        if (isset($row['keys']) && ($row['keys'] !== '') && isset($row['columns'])) {
            foreach ($row['keys'] as $keyColumn) {
                foreach ($row['columns'] as &$column) {
                    if ($column['name'] === $keyColumn) {
                        $column['isKey'] = true;
                    }
                }
            }
        }
    }

    /**
     * @param $row
     * @return array
     * @throws Exception
     * @throws \Exception
     * @throws \Zeedhi\Framework\ErrorHandler\ErrorException
     */
    protected function getColumns($row)
    {
        try {
            return $this->dataSourceService->getMergedColumns($row);
        } catch (\Service\Exception $e) {
            throw Exception::connectionNotFound($row['connectionName']);
        } catch (\Doctrine\DBAL\DBALException $e) {
            throw Exception::createConnectionError($row['connectionName']);
        } catch (\Zeedhi\Framework\ErrorHandler\ErrorException $e) {
            if (strrpos($e->getMessage(), self::ERRORS_CONNECT_FAIL)) {
                throw Exception::connectFail();
            } else {
                throw $e;
            }
        }
    }

    protected function getColumnsFromWebSocket($row)
    {
        try {
            $cleanedQuery = $this->dataSourceOtherService->replaceQueryParameters($row['query']);
            $organizationId = Utilities::getOrganizationIdForMongo();
            return $this->dataSourceService->getMergedColumnsFromWebSocket($this->socketClient, $cleanedQuery, $row, $organizationId);
        } catch (\Zeedhi\Framework\ErrorHandler\ErrorException $e) {
            if (strrpos($e->getMessage(), self::ERRORS_CONNECT_FAIL)) {
                throw Exception::connectFail();
            } else if (strrpos($e->getMessage(), self::ERRORS_EXECUTE_QUERY_FAIL)) {
                throw Exception::executeQueryError($row['query'], $e);
            } else {
                throw $e;
            }
        }
    }

    protected function getColumnsFromOriginDataSources($row)
    {
        $columns = array();

        foreach ($row['originDataSources'] as $originDataSourceId) {
            $originDataSource = $this->getDataSource($originDataSourceId);
            $this->dataSourceOtherService->getColumnsFromOriginDataSources($originDataSource, $columns);
        }
        return $columns;

    }

    protected function addHistoryDataToRows(&$rows)
    {
        $mapIds = function ($item) {
            return $item['_id'];
        };
        $dataSourceIds = array_map($mapIds, $rows);
        $historyItems = $this->importHistoryService->findLastOnesByDataSourceIds($dataSourceIds);
        $historyItems = $this->importHistoryService->prepareHistoryRows($historyItems);

        foreach ($rows as &$row) {
            if (isset($historyItems[$row['_id']]) && !isset($row['status'])) {
                $row['importStatus'] = $this->returnIfIsset($historyItems[$row['_id']], "status");
                $row['importHasError'] = $this->returnIfIsset($historyItems[$row['_id']], "hasError");
                $row['importSubprocesses'] = $this->returnIfIsset($historyItems[$row['_id']], "subprocesses");
                $row['totalImported'] = $this->returnIfIsset($historyItems[$row['_id']], "totalImported");
            }
        }
    }

    protected function returnIfIsset($arr, $property)
    {
        return isset($arr[$property]) ? $arr[$property] : null;
    }

    protected function setFactsJoinFunctionsInformations(&$joinDataSource)
    {

        try {
            $joinDataSourceColumns = $joinDataSource['columns'];

            foreach ($joinDataSource['originDataSources'] as $originDataSourceId) {
                $originDataSourceName = $this->getDataSource($originDataSourceId)['name'];
                $dataSourceFactsInformations = array();

                $factInformations = array();
                foreach ($joinDataSourceColumns as $column) {
                    $isColumnsGeneratedFact = $this->isColumnsGeneratedFact($column);
                    if ($isColumnsGeneratedFact && (isset($column['columnOriginDataSources']) && (strrpos($column['columnOriginDataSources'], $originDataSourceName) !== false))) {
                        $joinFunction = isset($column['joinFunction']) ? $column['joinFunction'] : $column['function'];
                        $factInformations[$column['name']] = $joinFunction;
                    }
                }
                $joinDataSource['dataSourcesInformations'][$originDataSourceId]['facts'] = $factInformations;
            }
        } catch (Exception $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        }
    }

    protected function getDataSource($dataSourceId)
    {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('_id', '=', $dataSourceId);
        $rows = $this->organizationMongoManager->findBy($filterCriteria)->getRows();
        $dataSource = reset($rows);
        return $dataSource;
    }

    protected function setAutoFactsTranslation(&$joinDataSource)
    {
        $factsTranslation = array();

        try {
            foreach ($joinDataSource['originDataSources'] as $originDataSourceId) {
                $dataSourceTranslations = array();

                $originDataSource = $this->getDataSource($originDataSourceId);

                if (isset($originDataSource['dateFilterColumn'])) {
                    $joinDataSource['dataSourcesInformations'][$originDataSourceId]['dateFilterColumn'] = $originDataSource['dateFilterColumn'];
                }

                $dataSourceTranslations = $this->translateFacts($joinDataSource['columns'], $originDataSource['columns']);
                $joinDataSource['dataSourcesInformations'][$originDataSourceId]['factsTranslation'] = $dataSourceTranslations;
            }
        } catch (Exception $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        }
    }

    protected function translateFacts($joinDataSourceColumns, $originDataSourceColumns)
    {
        $dataSourceFactsTranslations = array();

        foreach ($joinDataSourceColumns as $joinColumn) {
            foreach ($originDataSourceColumns as $originColumn) {
                $checkJoinColumn = $this->isColumnsGeneratedFact($joinColumn);
                $checkOriginColumn = $this->isColumnsGeneratedFact($originColumn);
                $columnName = $joinColumn['name'];

                if ($checkJoinColumn && $checkOriginColumn && $columnName === $originColumn['name']) {
                    $dataSourceFactsTranslations[$columnName] = array($columnName);
                }
            }
        }

        return $dataSourceFactsTranslations;
    }

    protected function isColumnsGeneratedFact($column)
    {
        $validation = false;

        if ((isset($column['isGenerated']) && $column['isGenerated']) &&
            (isset($column['isFact']) && $column['isFact'])
        ) {
            $validation = true;
        }

        return $validation;
    }

    public function dataSourceGeneratedData(DTO\Request\DataSet $request, DTO\Response $response)
    {
        ini_set("memory_limit", "1024M");
        $row = $request->getDataSet()->getRows()[0];
        $dataSourceId = $row['__META_DATA']['DATA_SOURCE_ID'];
        if (!empty($row['__META_DATA']['FIELD_NAMES']) && is_array($row['__META_DATA']['FIELD_NAMES'])) {
            $values = array();
            foreach ($row['__META_DATA']['FIELD_NAMES'] as $fieldNames) {
                if (!empty($row[$fieldNames])) {
                    $values[$fieldNames] = array(
                        $row[$fieldNames],
                        $row[$fieldNames . '___TYPE']
                    );
                }
            }
            $this->dataSourceOtherService->dataSourceGeneratedData($values, $dataSourceId);
        }
    }
}
