import { Metadata, Http, DateHelper, Loader } from '@zeedhi/core';
import { Modal, ModalService, Form, DialogService, LoadingService, Select, DateRange } from '@zeedhi/common';
import { Highlight } from '@zeedhi/zd-highlight-common';
import Prism from 'prismjs';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-php.min';
import { DataSourceService } from '@/Services/DataSourceService';
import { CustomAlertService } from '@/Services/CustomAlertService';
import { PeriodsService } from '@/Services/PeriodsService';
import { Utilities } from '@/Services/Utilities';
import { DataSourceController } from './DataSourceController';

export class DSImportScheduleController {
    private mainController: DataSourceController = Loader.getInstance("DataSourceController");
    public service = new DataSourceService();

    private get datasourceData() {
        return this.mainController.datasourceData;
    }

    public importNowModal!: Modal;
    public importNowFormMounted({ component }: any) {
        const period = component.getChildInstance('periodNow');
        // Diminui o tamanho do menu de data
        const periodDropdown = period.childrenInstances[0].childrenInstances[0];
        periodDropdown.maxHeight = 190;
        periodDropdown.value = this.datasourceData.periodsCode;
        Utilities.generatePeriodHelperOptions(period, this.datasourceData.period);
    }
    public closeImportNow() {
        this.importNowModal.hide();
        this.importNowModal.destroy();
    }
    public async importSchedulingNow() {
        this.importNowModal?.destroy();
        this.importNowModal = await ModalService.createFromJson('dataSource/ImportScheduling/importNowModal', true);
        this.importNowModal.show();
    }
    public async handleImportNow() {
        const importNowForm = Metadata.getInstance<Form>('importNowForm');
        const period = importNowForm.getChildInstance<DateRange>('periodNow');
        const data = this.generateImportScheduling({
            persistType: importNowForm.value.persistType,
            period: period.helperValue || importNowForm.value.period,
        });
        data.dataSource = this.datasourceData._id;
        data.status = 'queue';
        data.type = 'manualBI';
        if (!importNowForm.validate()) {
            return;
        }
        LoadingService.show();
        const { inProcess, processes } = await this.checkImportExistence();
        LoadingService.hide();
        if (inProcess) {
            DialogService.show({
                name: 'inProcess_dialog',
                type: 'warning',
                title: 'Atenção',
                text: 'Há um ou mais imports em progresso. O que você gostaria de fazer?',
                buttons: [
                    { name: 'replace', component: 'ZdButton', label: 'Substituir processos de importação', events: { click: () => { DialogService.hide(); this.replaceImportProcess(processes, data); }}, outline: true },
                    { name: 'add', component: 'ZdButton', label: 'Adicionar a fila', events: { click: () => { DialogService.hide(); this.addScheduleToQueue(data); }}, outline: true },
                    { name: 'cancel', component: 'ZdButton', label: 'Cancelar', events: { click: () => { DialogService.hide(); this.closeImportNow(); }} }
                ]
            });
        } else {
            await this.saveImportNowSchedule(data);
        }
    }
    public async checkImportExistence() {
        const res = await Http.post("/dataSource/findImports", {
			"requestType": "FilterData",
			"filter": [
				{
					"name": "dataSourceId",
					"operator": "=",
					"value": this.datasourceData._id
				}
			]
		});
        const processes = res.data.dataset;
        return {
            processes,
            inProcess: processes.totalImportsInProcess[0] > 0
        };
    }
    public async saveImportNowSchedule(importScheduling: any) {
        let status = false;
        let errorMessage = '';
        const res = await Http.post("/importSchedule/save", {
            "requestType": "Row",
            "row": { ...importScheduling }
        });
        if (res.data.notifications && res.data.notifications[0]) {
            const notification = res.data.notifications[0];
            if (notification.type === 'success') {
                status = true;
            } else {
                errorMessage = notification.message;
            }
        } else if (res.data.messages && res.data.messages[0]) {
            const message = res.data.messages[0];
            if (message.type === 'E') {
                errorMessage = message.message;
            } else {
                status = true;
            }
        }
        LoadingService.hide();
        if (status) {
            CustomAlertService.successAlert("Importation successfully scheduled! You can follow up the process at the tab 'Import History' in a few moments");
            this.closeImportNow();
        } else {
            CustomAlertService.errorAlert(errorMessage || "Um erro ocorreu ao realizar a importação");
        }
    }
    public async replaceImportProcess(processes: any, importScheduling: any) {
        if (processes.totalImportsBlocked[0] > 0) {
            CustomAlertService.errorAlert('IMPORT_IN_PROCESS');
        } else {
            LoadingService.show();
            await Http.post("/dataSource/cancelImportation", {
                "requestType": "FilterData",
                "filter": [
                    {
                        "name": "dataSourceId",
                        "operator": "=",
                        "value": this.datasourceData._id
                    }
                ]
            });
            await this.saveImportNowSchedule(importScheduling);
        }
    }
    public async addScheduleToQueue(scheduleItem: any) {
        LoadingService.show();
        const userRes = await Utilities.getLoggedUser();
        const { username } = userRes;
        scheduleItem.userResponsible = username;
        await Http.post("/manualImportQueue/save", {
			"requestType": "Row",
			"row": {
                'scheduleItem': scheduleItem
            }
		});
        LoadingService.hide();
        this.closeImportNow();
    }

    public generateImportScheduling(importNowParams: any = null) {
        const form = Metadata.getInstance<Form>('importScheduling');
        const afterImportScript = Metadata.getInstance<Highlight>('afterImportScript');
        const beforeImportScript = Metadata.getInstance<Highlight>('beforeImportScript');
        const period = form.getChildInstance<DateRange>('period');
        const values: any = {
            ...form.value,
            afterImportScript: afterImportScript.code,
            beforeImportScript: beforeImportScript.code,
            period: period.helperValue || (form.value.period ? { start: form.value.period[0], end: form.value.period[1] } : null),
            dateFilterColumn: form.value.dateFilterColumnLabel?.name,
            dateFilterColumnLabel: form.value.dateFilterColumnLabel?.label
        };
        if (importNowParams) {
            values.persistType = importNowParams.persistType || values.persistType;
            values.period = importNowParams.period || values.period;
        }
        return values;
    }

    public saveImportSchedulingFormConfirmationDialog() {
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Salvar agendamento de importação',
            text: 'Deseja salvar o agendamento de importação?',
            buttons: [
              { name: 'no', component: 'ZdButton', label: 'NO', events: {click: () => DialogService.hide()}, outline: true},
              { name: 'yes', component: 'ZdButton', label: 'YES', events: {click: () => this.saveImportSchedulingFormConfirm()}},
            ],
          });
    }

    private saveImportSchedulingFormConfirm() {
        DialogService.hide();
        this.saveImportScheduling();
    }

    public async saveImportScheduling() {
        const form = Metadata.getInstance<Form>('importScheduling');
        if (!form.validate()) {
            return;
        }
        const values = this.generateImportScheduling();
        console.log(values);
        Object.assign(this.datasourceData, values);
        LoadingService.show();
        try {
            const res = await this.service.saveDataSource(this.datasourceData, this.mainController.user.userId);
            CustomAlertService.saveSuccessAlert();
            if (res.data.messages?.length > 0) {
                const message = res.data.messages[0].message;
                DialogService.show({
                  name: 'warning-dialog',
                  type: 'error',
                  title: ' ',
                  text: message
                });
            }
            this.importSchedulingUpdateForm();
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }
    public async importSchedulingUpdateForm() {
        // Se formulário não foi montado, não faz nada
        if (Metadata.getInstances('importScheduling').length === 0 || !Metadata.getInstances('importScheduling')[0]) {
            return;
        }
        const form = Metadata.getInstance<Form>('importScheduling');
        const keys = form.getChildInstance<Select>('keys');
        const dateFilter = form.getChildInstance<Select>('dateFilterColumnLabel');
        const period = form.getChildInstance<DateRange>('period');
        const afterImportScript = Metadata.getInstance<Highlight>('afterImportScript');
        const beforeImportScript = Metadata.getInstance<Highlight>('beforeImportScript');
        keys.datasource.updateData(this.datasourceData.columns || []);
        dateFilter.datasource.updateData(this.datasourceData.columns?.filter((col: any) => col.isDate) || []);

        // Diminui o tamanho do menu de data
        const periodDropdown = period.childrenInstances[0].childrenInstances[0];
        periodDropdown.maxHeight = 190;
        await Utilities.generatePeriodHelperOptions(period, this.datasourceData.period);

        const ignoreKeys: string[] = ["period"];
        Object.keys(form.value).filter((key: string) => !ignoreKeys.includes(key)).forEach((key: any) => { form.value[key] = this.datasourceData[key]; });
        period.change();
        Utilities.setPeriodHelperValue(period, this.datasourceData.period);
        afterImportScript.code = this.datasourceData.afterImportScript || "";
        beforeImportScript.code = this.datasourceData.beforeImportScript || "";
    }
    public loadLanguageAfterImport() {
        const { code, language } = Metadata.getInstance<Highlight>('afterImportScript');
        return Prism.highlight(code, Prism.languages[language], language);
    }
    public loadLanguageBeforeImport() {
        const { code, language } = Metadata.getInstance<Highlight>('beforeImportScript');
        return Prism.highlight(code, Prism.languages[language], language);
    }
}
